import java.math.*;
import java.io.*;

	public class JigsawSudoku {
		public static void main(String args[]) {
			System.out.println("Jigsaw-Sudoku-Solver by Simon v1.1\n");
			//SudokuDatabase.selfTest();
			if (args.length<=3) {
				System.err.println("Useage:");
				System.err.println("(1) Check a Sudoku puzzle:");
				System.err.println("    SudokuSolver -c size sudoku_file_name block_file_name");
				System.err.println("");
				System.err.println("(2) Solve a Sudoku puzzle:");
				System.err.println("    SudokuSolver -s size sudoku_file_name block_file_name");
				System.err.println("");
				return;
			}
			int gsize=9;
			if (args.length==4) {
				gsize=Integer.valueOf(args[1]);
				System.out.println("Sudoku size: "+gsize);
				Sudoku.init(gsize);
	        }
			else {
				System.err.println("Insufficient parameters!");
				return;
			}
			if (args[0].equals("-s")) {
				if (args.length==4) {
					System.out.println("Input Sudoku file: "+args[args.length-2]);
					System.out.println("Input Block file: "+args[args.length-1]);
					if (gsize<=9) {
						if (!Sudoku.readFromFile(args[args.length-2], Sudoku.sp_init)) return;
						if (!Sudoku.readFromFile(args[args.length-1], Sudoku.blocknr)) return;
					}
					else {
						if (!Sudoku.readFromFileWide(args[args.length-2], Sudoku.sp_init)) return;
						if (!Sudoku.readFromFileWide(args[args.length-1], Sudoku.blocknr)) return;
					}
				}
			}
			else if (args[0].equals("-c")) {
				if (args.length==4) {
					System.out.println("Input file: "+args[args.length-1]);
					if (gsize<=9) {
						if (!Sudoku.readFromFile(args[args.length-2], Sudoku.sp_init)) return;
						if (!Sudoku.readFromFile(args[args.length-1], Sudoku.blocknr)) return;
					}
					else {
						if (!Sudoku.readFromFileWide(args[args.length-2], Sudoku.sp_init)) return;
						if (!Sudoku.readFromFileWide(args[args.length-1], Sudoku.blocknr)) return;
					}
				}
			}
			else {
				System.err.println("Mode parameter "+args[0]+" invalid!");
				return;
			}
			Sudoku start = new Sudoku(Sudoku.sp_init, "Init", 0);
			System.out.println("\nSudoku:");
			start.printSolution();
			if (args[0].equals("-s") || args[0].equals("-gs")) {
				start.work();
				System.out.print("done. ");
				if (Sudoku.solution>1)
					System.out.println(Sudoku.solution+" solutions were found!");
				else if (Sudoku.solution==1)
					System.out.println("1 solution was found!");
				else
					System.out.println("No solution!");
				if (Sudoku.maxLevel>0)
					System.out.println("I had to use backtracking with depth "+Sudoku.maxLevel);
			}
			else if (args[0].equals("-c")) {
				if (Sudoku.checkGame(Sudoku.sp_init, gsize))
					System.out.println("Sudoku seems valid.");
				else
					System.out.println("Sudoku invalid.");
			}
		}
	}


	class Sudoku {
		public static final boolean debug = false;

		public static int GAMESIZE = 64;
		public static int NUMBEROFFSET = 1;
		public static int solution=0;
		public static int maxLevel=0;
		public static final boolean ADVANCED = true;

		public static int[][] sp_init = new int[GAMESIZE][GAMESIZE];
	    public static int[][] blkx = new int[GAMESIZE][GAMESIZE], blky = new int[GAMESIZE][GAMESIZE], blocknr = new int[GAMESIZE][GAMESIZE];
	    private static int cx, cy;

		private int[][] sp = new int[GAMESIZE][GAMESIZE];	// Spielfeld
		private int[][] spcnt = new int[GAMESIZE][GAMESIZE];	// Anzahl Mglichkeiten
		private long[] rowid = new long[GAMESIZE], colid = new long[GAMESIZE], blockid = new long[GAMESIZE];
		private String desc;
		private int level;
		private boolean ungueltig=false;


		public Sudoku(int[][] gsp, String s, int l) {
			desc=s;
			level=l;
			int[] blkxcnt = new int[GAMESIZE], blkycnt = new int[GAMESIZE];
			for (int i=0; i<GAMESIZE; i++) {
				blkxcnt[i] = 0;
				blkycnt[i] = 0;
			}
			if (l>maxLevel) maxLevel=l;
			for (int x=0; x<GAMESIZE; x++) 
				for (int y=0; y<GAMESIZE; y++) {
					sp[y][x] = gsp[y][x];
					blkx[blocknr[y][x]][blkxcnt[blocknr[y][x]]++] = x;
					blky[blocknr[y][x]][blkycnt[blocknr[y][x]]++] = y;
				}
			for (int i=0; i<GAMESIZE; i++) {
				if (blkxcnt[i] != GAMESIZE) {System.err.println("Blockmap is invalid!"); return;}
				if (blkycnt[i] != GAMESIZE) {System.err.println("Blockmap is invalid!"); return;}
			}
	        if (debug && !checkGame(sp, GAMESIZE))
	            System.err.println("New game is invalid!");
		}


	    public static void init(int gsize) {
	        GAMESIZE=gsize;
	        solution=0;
	        sp_init=new int[GAMESIZE][GAMESIZE];
	    }
	    

	    public static boolean checkGame(int[][] game, int gsize) {
	        for (int y=0; y<gsize; y++) {
	            BigInteger c=BigInteger.ZERO;
	            for (int x=0; x<gsize; x++) {
	                int l=game[y][x];
	                if (l >= 0) {
	                    if (!c.testBit(l))
	                        c.setBit(l);
	                    else
	                        return false;
	                }
	            }
	        }
	        for (int x=0; x<gsize; x++) {
	            BigInteger c=BigInteger.ZERO;
	            for (int y=0; y<gsize; y++) {
	                int l=game[y][x];
	                if (l >= 0) {
	                    if (!c.testBit(l))
	                        c.setBit(l);
	                    else
	                        return false;
	                }
	            }
	        }
	        for (int blk=0; blk<gsize; blk++) {
	                BigInteger c=BigInteger.ZERO;
	                for (int el=0; el<gsize; el++) {
                        int l=game[blky[blk][el]][blkx[blk][el]];
                        if (l >= 0) {
                            if (!c.testBit(l))
                                c.setBit(l);
                            else
                                return false;
                        }
	                }
	        }
	        return true;
	    }
	    
	    
		public static boolean readFromFile(String fn, int[][] dest) {
			File f = new File(fn);
			int cnt=0, o=0;
			char[] buffer = new char[(int)f.length()];
			try {
				InputStream in = new FileInputStream(f);
				while (cnt<f.length()) {
					char b = (char)in.read();
					if ((b>='1' && b<='9') || (b>='a' && b<='f') || (b>='A' && b<='F') || b == '.' || b == '*')
						buffer[o++] = b;
					cnt++;
				}
			} catch (Exception e) {System.err.println("Read error from file "+fn); return false;}
			if (o==0 || o>GAMESIZE*GAMESIZE) {
				System.err.println("Input is invalid!");
				return false;
			}
			for (int y=0; y<GAMESIZE; y++)
				for (int x=0; x<GAMESIZE; x++) {
					if (buffer[x+y*GAMESIZE] != '.' && buffer[x+y*GAMESIZE] != '*') {
						String s=""+buffer[x+y*GAMESIZE];
						dest[y][x]=Integer.parseInt(s, 16)-NUMBEROFFSET;
					}
					else
						dest[y][x]=-1;
				}
			return true;
		}
		
		
		private static int eatNumber(InputStream in) {
			int t;
			String s="";
			try {
				do
					t = in.read();
				while (t != -1 && (t<(int)('0') || t>(int)('9')) && t!=(int)('*') && t!=(int)('.'));
				if (t == -1) return -2;
				while (t != -1 && (t>=(int)('0') && t<=(int)('9')) || t==(int)('.') || t==(int)('*')) {
					if (t>0) s+=(char)t;
					t = in.read();
				}
				if (t == -1) return -2;
				if (s.equals("*") || s.equals(".")) return -1;
				else return Integer.parseInt(s)-NUMBEROFFSET;
			} catch (Exception e) {System.err.println("Error reading from file.");}
			return -1;
		}
		
		public static boolean readFromFileWide(String fn, int[][] dest) {
			File f = new File(fn);
			int o=0;
			int[] buffer = new int[(int)f.length()];
			try {
				InputStream in = new FileInputStream(f);
				while (true) {
					int t = eatNumber(in);
					if (t >= -1) buffer[o++] = t;
					else break;
				}
			} catch (Exception e) {System.err.println("Read error from file "+fn); return false;}
			if (o==0 || o>GAMESIZE*GAMESIZE) {
				System.err.println("Input is invalid!");
				return false;
			}
			for (int y=0; y<GAMESIZE; y++)
				for (int x=0; x<GAMESIZE; x++)
					dest[y][x]=buffer[x+y*GAMESIZE];
			return true;
		}
		
		
	    private boolean checkRow(int row) {
	        int[] anz = new int[GAMESIZE], lpos = new int[GAMESIZE];
	        boolean c=false;
	        for (int i=0; i<GAMESIZE; i++) {anz[i] = 0; lpos[i] = 0;}
	        for (int x=0; x<GAMESIZE; x++) if (spcnt[row][x]>1) {
	            long temp = schnittmenge(row, x);
	            for (int i=0; i<GAMESIZE; i++)
	                if ((temp&(1L<<(long)i))!=0) {anz[i]++; lpos[i]=x;}
	        }
	        for (int i=0; i<GAMESIZE; i++)
	            if (anz[i] == 1) {sp[row][lpos[i]] = i; spcnt[row][lpos[i]]=1; cy=row; cx=lpos[i]; return true;}
	        return c;
	    }
	    
	    
	    private boolean checkCol(int col) {
	        int[] anz = new int[GAMESIZE], lpos = new int[GAMESIZE];
	        boolean c=false;
	        for (int i=0; i<GAMESIZE; i++) {anz[i] = 0; lpos[i] = 0;}
	        for (int y=0; y<GAMESIZE; y++) if (spcnt[y][col]>1) {
	            long temp = schnittmenge(y, col);
	            for (int i=0; i<GAMESIZE; i++)
	                if ((temp&(1L<<(long)i))!=0) {anz[i]++; lpos[i]=y;}
	        }
	        for (int i=0; i<GAMESIZE; i++)
	            if (anz[i] == 1) {sp[lpos[i]][col] = i; spcnt[lpos[i]][col]=1; cy=lpos[i]; cx=col; return true;}
	        return c;
	    }
	    
	    
	    private boolean checkBlock(int blk) {
	        int[] anz = new int[GAMESIZE], lposx = new int[GAMESIZE], lposy = new int[GAMESIZE];
	        boolean c=false;
	        for (int i=0; i<GAMESIZE; i++) {anz[i] = 0; lposx[i] = 0; lposy[i] = 0;}
	        for (int el=0; el<GAMESIZE; el++) {
	            if (spcnt[blky[blk][el]][blkx[blk][el]]>1) {
	                long temp = schnittmenge(blky[blk][el], blkx[blk][el]);
	                for (int i=0; i<GAMESIZE; i++)
	                    if ((temp&(1L<<(long)i))!=0) {anz[i]++; lposx[i]=blkx[blk][el]; lposy[i]=blky[blk][el];}
	            }
	        }
	        for (int i=0; i<GAMESIZE; i++)
	            if (anz[i] == 1) {sp[lposy[i]][lposx[i]] = i; spcnt[lposy[i]][lposx[i]]=1; cy=lposy[i]; cx=lposx[i]; return true;}
	        return c;
	    }
	    
	    
		private void remrowid(int row, int id) {
			if (id<0 || id>GAMESIZE-1 || row<0 || row>GAMESIZE-1) 
				return;
			rowid[row] &= ~(long)(1L<<(long)id);
		}


		private void remcolid(int col, int id) {
			if (id<0 || id>GAMESIZE-1 || col<0 || col>GAMESIZE-1) 
				return;
			colid[col] &= ~(long)(1L<<(long)id);
		}


		private void remblockid(int blk, int id) {
			if (id<0 || id>GAMESIZE-1 || blk<0 || blk>GAMESIZE-1) 
				return;
			blockid[blk] &= ~(long)(1L<<(long)id);
		}


		private static int getBlock(int row, int col) {
			return blocknr[row][col];
		}


		private long schnittmenge(int row, int col) {
			int blk=getBlock(row, col);
			if (row<0 || row>GAMESIZE-1 || col<0 || col>GAMESIZE-1) 
				return -1L;
			return rowid[row] & colid[col] & blockid[blk];
		}


		private static int countbits(long id) {
			int erg=0;
			for (int i=0; i<GAMESIZE; i++)
				if (((long)id&(long)(1L<<(long)i)) != 0L) 
					erg++;
			return erg;
		}


		private static int bitnummer(long id) {
			if (id==0) return -1;
			int erg=0;
			while (((long)id&1L) == 0L) {
				id=id>>>1L;
				erg++;
			}
			return erg;
		}
		
		
		private static String sprintPool(long d) {
			String s="";
	        int erg=0;
	        while (d != 0L && erg < GAMESIZE) {
	            if ((d&1L) == 1L)
	                s=s + (erg+NUMBEROFFSET) + ", ";
	            d>>>=1L;
	            erg++;
	        }
	        return s.substring(0,s.length()-2);
		}
	    
	    
		private static void printSeparator(int row) {
			String fmt1="   ";
			String fmt2="---";
			int last=1;
			if (GAMESIZE >= 1000) {fmt1="      ";fmt2="------";}
			else if (GAMESIZE >= 100) {fmt1="     ";fmt2="-----";}
			else if (GAMESIZE >= 10) {fmt1="    ";fmt2="----";}
			for (int c=0; c<GAMESIZE; c++) {
				if (row==0 || (row>0 && blocknr[row-1][c]!=blocknr[row][c])) {
					System.out.print("+"+fmt2);
					last=1; 
				}
				else {
					if (c==0) 
						System.out.print("|"+fmt1);
					else if (last==1) 
						System.out.print("+"+fmt1);
					else if (last==0) {
						if (blocknr[row][c-1]==blocknr[row][c])
							System.out.print(" "+fmt1);
						else
							System.out.print("|"+fmt1);
					}
					last=0;
				}
			}
			if (last==0 && row!=0) 
				System.out.println("|"); 
			else 
				System.out.println("+");
		}
		
		
		public void printSolution() {
			int fc=0;
			String fmt="%1d", fmt2="*";
			if (GAMESIZE >= 1000) {fmt="%4d"; fmt2="   *";}
			else if (GAMESIZE >= 100) {fmt="%3d"; fmt2="  *";}
			else if (GAMESIZE >= 10) {fmt="%2d"; fmt2=" *";}
			
			for (int y=0; y<GAMESIZE; y++) {
				printSeparator(y);
				for (int x=0; x<GAMESIZE; x++) {
					if (x==0 || (x>0 && blocknr[y][x-1]!=blocknr[y][x]))
						System.out.print("|");
					else
						System.out.print(" ");
					if (sp[y][x]>=0) {
						String s=String.format(fmt, sp[y][x]+NUMBEROFFSET);
						System.out.print(" "+s+" ");
					}
					else {
						System.out.print(" "+fmt2+" ");
						fc++;
					}
				}
				System.out.print("|");
				System.out.println("");
			}
			printSeparator(0);
			if (fc>0)
				System.out.println("Unknown numbers: "+fc+"\n");
			else
				System.out.println("");
		}


		private boolean berechne() {
			boolean change, done;
			do {
				done=true;
				change=false;
				for (int i=0; i<GAMESIZE; i++) {
					rowid[i]=0xffffffffffffffffL;
					colid[i]=0xffffffffffffffffL;
					blockid[i]=0xffffffffffffffffL;
				}
				for (int y=0; y<GAMESIZE; y++)
					for (int x=0; x<GAMESIZE; x++)
						if (sp[y][x]>=0) 
							spcnt[y][x]=1;
						else 
							spcnt[y][x]=GAMESIZE;
				for (int y=0; y<GAMESIZE; y++)
					for (int x=0; x<GAMESIZE; x++) 
						remrowid(y, sp[y][x]);
				for (int x=0; x<GAMESIZE; x++) 
					for (int y=0; y<GAMESIZE; y++)
						remcolid(x, sp[y][x]);
				for (int blk=0; blk<GAMESIZE; blk++) 
					for (int el=0; el<GAMESIZE; el++)
						remblockid(blk, sp[blky[blk][el]][blkx[blk][el]]);
				if (ADVANCED) {
					for (int y=0; y<GAMESIZE; y++) {
						change = checkRow(y);
						if (change)
							break;
					}
					if (!change) for (int x=0; x<GAMESIZE; x++) {
						change = checkCol(x);
						if (change)
							break;
					}
					if (!change) for (int b=0; b<GAMESIZE; b++) {
						change = checkBlock(b);
						if (change)
							break;
					}
				}
				if (!change) 
	              for (int y=0; y<GAMESIZE; y++) {
					for (int x=0; x<GAMESIZE; x++) {
						if (spcnt[y][x]>1) {
							int t=countbits(schnittmenge(y, x));
							if (t==0) {
								if (debug) {
									String s="";
									for (int c=0; c<level; c++)
										s=s+" ";
									System.out.println(s+"-> w1 Failed ("+desc.trim()+")");
								}
								return false;
							}
							else if (t==1) {
								sp[y][x]=bitnummer(schnittmenge(y,x));
	                            spcnt[y][x]=1;
								change=true; break;
							}
							else 
								done=false;
							spcnt[y][x]=t;
						}
					}
					if (change) 
						break;
				}
			}
			while (change);
			if (done) {
				solution++;
				System.out.println("Solution "+solution+":");
				if (debug && checkGame(sp, GAMESIZE)) System.out.println("<<Solution valid>>");
				printSolution();
			}
			return true;
		}


		private boolean berechne(int gx, int gy) {
			boolean change, done;
			do {
				done=true;
				change=false;
	            remrowid(gy, sp[gy][gx]);
	            remcolid(gx, sp[gy][gx]);
	            remblockid(getBlock(gy, gx), sp[gy][gx]);
	            if (ADVANCED) {
	            	change=checkRow(gy); if (change) {gy=cy; gx=cx;}
	            	if (!change) {change=checkCol(gx); if (change) {gy=cy; gx=cx;}}
	            	if (!change) {change=checkBlock(getBlock(gy, gx)); if (change) {gy=cy; gx=cx;}}
	            }
				if (!change) for (int y=0; y<GAMESIZE; y++) {
					for (int x=0; x<GAMESIZE; x++) {
						if (spcnt[y][x]>1) {
							int t=countbits(schnittmenge(y,x));
							if (t==0) {
								if (debug) {
									String s="";
									for (int c=0; c<level; c++)
										s=s+" ";
									System.out.println(s+"-> w2 Failed ("+desc.trim()+")");
								}
								return false;
							}
							else if (t==1) {
								sp[y][x]=bitnummer(schnittmenge(y,x)); 
	                            spcnt[y][x]=1;
	                            gy=y; gx=x;
								change=true; break;
							}
							else 
								done=false;
							spcnt[y][x]=t;
						}
					}
					if (change) 
						break;
				}
			}
			while (change);
			if (done) {
				solution++;
				System.out.println("Solution "+solution+":");
				if (debug && checkGame(sp, GAMESIZE)) System.out.println("<<Solution valid>>");
				printSolution();
			}
			return true;
		}


		public void work() {
			if (!berechne()) {
				if (ungueltig)
					System.out.println("Input data invalid!");
				return;
			}
			int mx=0, my=0, minsp=GAMESIZE;
			for (int y=0; y<GAMESIZE; y++) {
				for (int x=0; x<GAMESIZE; x++) {
					if (spcnt[y][x]>1 && spcnt[y][x]<minsp) {
						mx=x; my=y; minsp=spcnt[y][x];
	                    if (minsp==2) break;
					}
				}
	            if (minsp==2) break;
			}
			long temp=schnittmenge(my,mx);
			for (int i=0; i<GAMESIZE; i++) 
				if (((temp>>>i)&1)==1)
				{
					sp[my][mx]=i;
					String s="";
					if (debug) {
						for (int c=0; c<level; c++)
							s=s+" ";
						s=s+"Start new w1@"+mx+","+my+" with "+(i+NUMBEROFFSET)+" of ["+sprintPool(temp)+"]";
						System.out.println(s);
					} 
					else s="";
					Sudoku next = new Sudoku(sp, s, level+1);
					next.work(spcnt, rowid, colid, blockid, mx, my);
					//if (ungueltig) return;
				}
		}
	    
	    
		public void work(int[][] gspcnt, long[] grow, long[] gcol, long[] gblk, int gx, int gy) {
	        for (int y=0; y<GAMESIZE; y++) {
	            for (int x=0; x<GAMESIZE; x++) {
	                spcnt[y][x] = gspcnt[y][x];
	            }
	            colid[y]=gcol[y];
	            rowid[y]=grow[y];
	            blockid[y]=gblk[y];
	        }
	        spcnt[gy][gx]=1;
	        
			if (!berechne(gx, gy)) {
				if (ungueltig)
					System.out.println("Input data invalid!");
				return;
			}
			int mx=0, my=0, minsp=GAMESIZE;
			for (int y=0; y<GAMESIZE; y++) {
				for (int x=0; x<GAMESIZE; x++) {
					if (spcnt[y][x]>1 && spcnt[y][x]<minsp) {
						mx=x; my=y; minsp=spcnt[y][x];
	                    if (minsp==2) break;
					}
				}
	            if (minsp==2) break;
			}
			long temp=schnittmenge(my,mx);
			for (int i=0; i<GAMESIZE; i++) 
				if (((temp>>>i)&1)==1)
				{
					sp[my][mx]=i;
					String s="";
					if (debug) {
						for (int c=0; c<level; c++)
							s=s+" ";
						s=s+"Start new w2@"+mx+","+my+" with "+(i+NUMBEROFFSET)+" of ["+sprintPool(temp)+"]";
						System.out.println(s);
					} 
					else s="";
					Sudoku next = new Sudoku(sp, s, level+1);
					next.work(spcnt, rowid, colid, blockid, mx, my);
					//if (ungueltig) return;
				}
		}
	}

